/*
	simil - This is my hack at the Ratcliff/Obershelp Pattern
	Recognition Algorithm as described in the July 1988 issue
	of Dr. Dobbs Journal. This algorithm differs from strcmp in
	that it returns a measure in percentage about how similar
	two strings are. This rendition is purely in C for portability
	reasons. The original was published in assembler for the small
	memory model of the 386. The original used home brew stacks
	for working space while this version handles everything by
	recursion.

	Copyright 1999  Steve Grubb  <linux_4ever@yahoo.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published  by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

 */

#include <string.h>		/* For memcmp()                 */
#include "Simil.h"

static float tcnt;

typedef struct {
	int o1;
	int o2;
	int len;
} SubString;

/*
*	This function takes 2 string pointers and their length and compares the
*	strings to find the biggest substring common to both strings. It puts the
*	size of the common substring and the offsets to it in a structure.
*/
static void
find_biggest_substring (char *s1, int s1len, char *s2, int s2len, SubString * ss)
{
	int i, j, size = 1;

	ss->o2 = -1;
	for (i = 0; i <= (s1len - size); i++) {
		for (j = 0; j <= (s2len - size); j++) {
			int test_size = size;

			while (1) {
				if ((test_size <= (s1len - i)) && (test_size <= (s2len - j))) {	// while things match, keep trying...
					if (memcmp (s1 + i, s2 + j, test_size) == 0) {
						if ((test_size > size)
						    || (ss->o2 < 0)) {
							ss->o1 = i;
							ss->o2 = j;
							size = test_size;
						}
						test_size++;
					} else	// Not equal
						break;
				} else	// Gone past the end of a string...we're done.
					break;
			}
		}
	}
	if (ss->o2 < 0)
		ss->len = 0;
	else
		ss->len = size;
}

/*
*	This function does the recursive work
*/
static void
rsimil (char *s1, int s1len, char *s2, int s2len)
{
	SubString ss;

	if ((s1len == 0) || (s2len == 0))
		return;

	find_biggest_substring (s1, s1len, s2, s2len, &ss);
	if (ss.len) {
		int delta1, delta2;
		tcnt += ss.len << 1;

		// Check left half...
		(void) rsimil (s1, ss.o1, s2, ss.o2);

		// Check right half...
		delta1 = ss.o1 + ss.len;
		delta2 = ss.o2 + ss.len;
		if ((delta1 < s1len) && (delta2 < s2len))
			rsimil (s1 + delta1, s1len - delta1, s2 + delta2, s2len - delta2);
	}
}

float
ratcliff2 (char *s1, char *s2)
{
	short s1len, s2len;
	float tlen, res;

	// preliminary tests
	res = strings_check (s1, s2);
	if (res != COMPARE)
		return (res);

	s1len = strlen (s1);
	s2len = strlen (s2);

	tcnt = 0;
	tlen = s1len + s2len;

	rsimil (s1, s1len, s2, s2len);

	return tcnt / tlen;
}
